(* ---------------------------------------------------*)
(*                 LISTMOD.MOD                        *)
(*   Allgemeingltigen Listenverwaltung in Modula 2   *)

IMPLEMENTATION MODULE ListMod;

FROM SYSTEM IMPORT ADDRESS, NULL;
FROM Storage IMPORT  ALLOCATE, DEALLOCATE;

TYPE LeafPtrType = POINTER TO LeafRecType;
     LeafRecType = RECORD
                     Next, Prev : LeafPtrType;
                     Data       : ADDRESS;
                   END;
     ListType = POINTER TO ListRecType;
     ListRecType = RECORD
                     Head, Tail, Current : LeafPtrType;
                     ListLength, RecLength : CARDINAL;
                   END;

VAR toAktList : ListType;


PROCEDURE LIstOpen (VAR L : ListType; Rec : CARDINAL);
BEGIN
  NEW (L);
  WITH L^ DO
    NEW (Head); Head^.Data:=NIL; Head^.Prev:=NIL;
    NEW (Tail); Tail^.Data:=NIL; Tail^.Next:=NIL;
    Head^.Next:=Tail; Tail^.Prev:=Head;
    Current:=Head;
    RecLength:=Rec;   ListLength:=0;
  END;  (* WITH L .. *)
END LIstOpen;

PROCEDURE LIstClose (VAR L : ListType);
BEGIN
  WITH L^ DO
    LIstHead(L);
    WHILE L^.Current<>L^.Tail DO
      LIstDelete(L);
    END;
    DISPOSE (Head);  DISPOSE (Tail);
    Head:=NIL;       Tail:=NIL;        Current:=NIL;
    RecLength:=0;    ListLength:=0;
  END;  (* WITH L .. *)
  DISPOSE (L);
END LIstClose;

PROCEDURE EmptyLIst (L : ListType) : BOOLEAN;
BEGIN  RETURN L^.Head^.Next=L^.Tail;  END EmptyLIst;

PROCEDURE HeadOfLIst (L : ListType) : BOOLEAN;
BEGIN  RETURN L^.Current=L^.Head;  END HeadOfLIst;

PROCEDURE TailOfLIst (L : ListType) : BOOLEAN;
BEGIN  RETURN L^.Current=L^.Tail;  END TailOfLIst;

PROCEDURE LengthOfLIst (L : ListType) : CARDINAL;
BEGIN  RETURN L^.ListLength;  END LengthOfLIst;

PROCEDURE LIstHead (L : ListType);
BEGIN  L^.Current:=L^.Head^.Next;  END LIstHead;

PROCEDURE LIstTail (L : ListType);
BEGIN  L^.Current:=L^.Tail^.Prev;  END LIstTail;

PROCEDURE LIstSavePosition (L : ListType;
                            VAR aktPointer : ADDRESS);
BEGIN  aktPointer:=L^.Current;  END LIstSavePosition;

PROCEDURE LIstLoadPosition (L : ListType;
                            aktPointer : ADDRESS);
BEGIN  L^.Current:=aktPointer;  END LIstLoadPosition;

PROCEDURE LIstAppend (L : ListType; Info : ADDRESS);
BEGIN
  WITH L^ DO
    NEW (Current);
    Current^.Data:=Info;
    Current^.Next:=Tail;      Current^.Prev:=Tail^.Prev;
    Tail^.Prev^.Next:=Current;   Tail^.Prev:=Current;
    INC(ListLength);
  END;  (* WITH L .. *)
END LIstAppend;

PROCEDURE LIstInsert (L : ListType; Info: ADDRESS);
VAR h : LeafPtrType;
BEGIN
  IF HeadOfLIst(L) THEN LIstNext(L); END;
  WITH L^ DO
    NEW (h);
    h^.Data:=Info;
    h^.Next:=Current;        h^.Prev:=Current^.Prev;
    Current^.Prev^.Next:=h;  Current^.Prev:=h;
    Current:=h;
    INC(ListLength);
  END;  (* WITH L .. *)
END LIstInsert;

PROCEDURE LIstDelete (L : ListType);
VAR h : LeafPtrType;
BEGIN
  IF L^.Current^.Data<>NULL THEN
    WITH L^ DO
      DEALLOCATE (Current^.Data,LONG(RecLength));
      h:=Current^.Prev;
      h^.Next:=Current^.Next;  Current^.Next^.Prev:=h;
      DISPOSE (Current);
      Current:=h^.Next;
      DEC(ListLength);
    END;  (* WITH L .. *)
  END;  (* IF Current .. *)
END LIstDelete;

PROCEDURE LIstCurrent (L : ListType; VAR Info: ADDRESS);
BEGIN  Info:=L^.Current^.Data;  END LIstCurrent;

PROCEDURE LIstNext (L : ListType);
BEGIN
  WITH L^ DO
    IF NOT TailOfLIst(L) THEN
      Current:=Current^.Next;
    END;
  END;  (* WITH L .. *)
END LIstNext;

PROCEDURE LIstPrev (L : ListType);
BEGIN
  WITH L^ DO
    IF NOT HeadOfLIst(L) THEN
      Current:=Current^.Prev;
    END;
  END;  (* WITH L .. *)
END LIstPrev;

PROCEDURE LIstSearchToTail (L        : ListType;
                            Position : ListPositionProc;
                            Vergleich: ADDRESS;
                            VAR gefunden  : BOOLEAN);
BEGIN
  WITH L^ DO
    WHILE NOT ((Current=Tail)
               OR Position (Vergleich,Current^.Data)) DO
      Current:=Current^.Next;
    END;  (* WHILE NOT .. *)
    gefunden:=NOT TailOfLIst(L);
  END;  (* WITH L .. *)
END LIstSearchToTail;

PROCEDURE LIstSearchToHead (L        : ListType;
                            Position : ListPositionProc;
                            Vergleich: ADDRESS;
                            VAR gefunden  : BOOLEAN);
BEGIN
  WITH L^ DO
    WHILE NOT ((Current=Head)
               OR Position (Vergleich,Current^.Data)) DO
      Current:=Current^.Prev;
    END;  (* WHILE NOT .. *)
    gefunden:=NOT HeadOfLIst(L);
  END;  (* WITH L .. *)
END LIstSearchToHead;

PROCEDURE LIstFromHeadToTail(L: ListType;
                         ListTraverse:ListTraverseProc);
VAR Cur : LeafPtrType;
BEGIN
  WITH L^ DO
    Cur:=Head^.Next;
    WHILE Cur<>Tail DO
      ListTraverse (Cur^.Data);
      Cur:=Cur^.Next;
    END;  (* WHILE NOT .. *)
  END;  (* WITH L .. *)
END LIstFromHeadToTail;

PROCEDURE LIstFromTailToHead(L: ListType;
                         ListTraverse:ListTraverseProc);
VAR Cur : LeafPtrType;
BEGIN
  WITH L^ DO
    Cur:=Tail^.Prev;
    WHILE Cur<>Head DO
      ListTraverse (Cur^.Data);
      Cur:=Cur^.Prev;
    END;  (* WHILE NOT .. *)
  END;  (* WITH L .. *)
END LIstFromTailToHead;

PROCEDURE ListDefault (VAR L : ListType);
BEGIN  toAktList:=L;  END ListDefault;

PROCEDURE ListOpen (VAR L : ListType; Rec : CARDINAL);
BEGIN  LIstOpen(L,Rec); ListDefault (L);  END ListOpen;

PROCEDURE ListClose;
BEGIN
  LIstClose(toAktList); toAktList:=NIL;
END ListClose;

PROCEDURE EmptyList () : BOOLEAN;
BEGIN  RETURN EmptyLIst(toAktList);  END EmptyList;

PROCEDURE HeadOfList () : BOOLEAN;
BEGIN  RETURN HeadOfLIst(toAktList);  END HeadOfList;

PROCEDURE TailOfList () : BOOLEAN;
BEGIN  RETURN TailOfLIst(toAktList);  END TailOfList;

PROCEDURE LengthOfList() : CARDINAL;
BEGIN  RETURN LengthOfLIst(toAktList); END LengthOfList;

PROCEDURE ListHead;
BEGIN  LIstHead(toAktList);  END ListHead;

PROCEDURE ListTail;
BEGIN  LIstTail(toAktList);  END ListTail;

PROCEDURE ListSavePosition (VAR aktPointer : ADDRESS);
BEGIN
  LIstSavePosition(toAktList,aktPointer);
END ListSavePosition;

PROCEDURE ListLoadPosition (aktPointer : ADDRESS);
BEGIN
  LIstSavePosition(toAktList,aktPointer);
END ListLoadPosition;

PROCEDURE ListAppend (Info : ADDRESS);
BEGIN  LIstAppend(toAktList,Info);  END ListAppend;

PROCEDURE ListInsert (Info: ADDRESS);
BEGIN  LIstInsert(toAktList,Info);  END ListInsert;

PROCEDURE ListDelete;
BEGIN  LIstDelete(toAktList);  END ListDelete;

PROCEDURE ListCurrent (VAR Info : ADDRESS);
BEGIN  LIstCurrent(toAktList,Info);  END ListCurrent;

PROCEDURE ListNext;
BEGIN  LIstNext(toAktList);  END ListNext;

PROCEDURE ListPrev;
BEGIN  LIstPrev(toAktList);  END ListPrev;

PROCEDURE ListSearchToTail (Position : ListPositionProc;
                            Vergleich : ADDRESS;
                            VAR gefunden  : BOOLEAN);
BEGIN
  LIstSearchToTail(toAktList,Position,
                   Vergleich,gefunden);
END ListSearchToTail;

PROCEDURE ListSearchToHead (Position : ListPositionProc;
                            Vergleich: ADDRESS;
                            VAR gefunden  : BOOLEAN);
BEGIN
  LIstSearchToHead(toAktList,Position,
                   Vergleich,gefunden);
END ListSearchToHead;

PROCEDURE ListFromHeadToTail(
                         ListTraverse:ListTraverseProc);
BEGIN
  LIstFromHeadToTail(toAktList,ListTraverse);
END ListFromHeadToTail;

PROCEDURE ListFromTailToHead (
                       ListTraverse : ListTraverseProc);
BEGIN
  LIstFromTailToHead(toAktList,ListTraverse);
END ListFromTailToHead;

BEGIN
  toAktList:=NIL;
END ListMod.
